@@ -107,7 +107,7 @@ The kernel module must set variables defining the kernel meta data:
     cylinder models).  The expression can call C functions, including
     those defined in your model file.
 
-A :class:`modelinfo.ModelInfo` structure is constructed from the kernel meta
+A :class:`.modelinfo.ModelInfo` structure is constructed from the kernel meta
 data and returned to the caller.
 
 Valid inputs should be identified by the *valid* expression.
@@ -142,10 +142,10 @@ Code follows the C99 standard with the following extensions and conditions::
     FLOAT_SIZE is the number of bytes in the converted variables
 
 :func:`load_kernel_module` loads the model definition file and
-:func:`modelinfo.make_model_info` parses it. :func:`make_source`
+:func:`.modelinfo.make_model_info` parses it. :func:`make_source`
 converts C-based model definitions to C source code, including the
 polydispersity integral.  :func:`model_sources` returns the list of
-source files the model depends on, and :func:`timestamp` returns
+source files the model depends on, and :func:`ocl_timestamp` returns
 the latest time stamp amongst the source files (so you can check if
 the model needs to be rebuilt).
 
@@ -175,7 +175,8 @@ from .custom import load_custom_kernel_module
 
 # pylint: disable=unused-import
 try:
-    from typing import Tuple, Sequence, Iterator, Dict
+    from typing import Tuple, Sequence, Iterator, Dict, List
+    from types import ModuleType
     from .modelinfo import ModelInfo
 except ImportError:
     pass
@@ -372,6 +373,7 @@ def dll_timestamp(model_info):
     changed file or dependency.
     """
     # TODO: what about on-the-fly derived models---tag name with crc32
+    # TODO: no longer used, and appears to be identical to ocl_timestamp
     # TODO: fails DRY; templates appear two places.
     model_templates = [joinpath(DATA_PATH, filename)
                        for filename in ('kernel_header.c', 'kernel_iq.c')]
@@ -457,8 +459,8 @@ def _convert_type(source, type_name, constant_flag):
     """
     # Convert double keyword to float/long double/half.
     # Accept an 'n' # parameter for vector # values, where n is 2, 4, 8 or 16.
-    # Assume complex numbers are represented as cdouble which is typedef'd
-    # to double2.
+    # Assume complex numbers are represented as cdouble.
+    # TODO: use lookahead so that "typedef complex double cdouble;" converts.
     source = re.sub(r'(^|[^a-zA-Z0-9_]c?)double(([248]|16)?($|[^a-zA-Z0-9_]))',
                     r'\1%s\2'%type_name, source)
     source = _tag_float(source, constant_flag)
@@ -706,7 +708,7 @@ def _split_translation(translation):
         # split assignment on first equals
         parts = code.split('=', 1)
         if len(parts) == 1:
-            raise ValueError("translation expected 'var=expr' but got %r", line)
+            raise ValueError("translation expected 'var=expr' but got %r"%line)
         # store parts without assessing whether they are correct
         var, expr = parts
         assignments.append((var.strip(), expr.strip()))
@@ -734,13 +736,13 @@ def _build_translation(model_info, table_id="_v", var_prefix="_var_"):
 
     * *subs* = {name: expr, ...} parameter substitution table for calling
       into the kernel function
-    * *vars* = "#define TRANSLATION_VARS(_v) _var_name = expr ...."
+    * *translation* = "#define TRANSLATION_VARS(_v) _var_name = expr ...."
     * *validity* = "#define VALID(_v) ..."
 
     The returned *subs* is used to generate the substitions for CALL_VOLUME
     etc., parameters, via :func:`_call_pars`.
 
-    The returned *vars* and *validity* macros need to be included inside
+    The returned *translation* and *validity* macros need to be included inside
     the generated model files.  They are the same for all variants (1D, 2D,
     magnetic) so can be defined once along side the parameter table.  Even
     though they are expanded independently in each variant.
@@ -770,15 +772,15 @@ def _build_translation(model_info, table_id="_v", var_prefix="_var_"):
     base_table = model_info.base.kernel_parameters
     call_pars = set(p.id for p in call_table)
     base_pars = set(p.id for p in base_table)
-    vars = set(name for name, expr in assigns
-               if name not in call_pars and name not in base_pars)
+    variables = set(name for name, expr in assigns
+                    if name not in call_pars and name not in base_pars)
 
     # Regular expression substition to tag the variables on each RHS.
     # Don't want to translate all symbols since they include C constants and
     # math functions.
     def id_sub(match_obj):
         name = match_obj.group(0)
-        if name in vars:
+        if name in variables:
             return var_prefix + name
         if name in call_pars:
             return table_id + '.' + name
@@ -791,7 +793,8 @@ def _build_translation(model_info, table_id="_v", var_prefix="_var_"):
     subs.update((name, eq) for name, eq in assigns if name in base_pars)
 
     # Build TRANSLATION_VARS macros assignments to variables.
-    var_values = [(var_prefix+name, eq) for name, eq in assigns if name in vars]
+    var_values = [(var_prefix+name, eq) for name, eq in assigns
+                  if name in variables]
     translation_vars = _build_translation_vars(table_id, var_values)
 
     # Build VALID macro, with expressions using caller parameters.
@@ -799,7 +802,7 @@ def _build_translation(model_info, table_id="_v", var_prefix="_var_"):
 
     return subs, translation_vars, validity
 
-def _build_translation_vars(table_id, vars):
+def _build_translation_vars(table_id, variables):
     r"""
     Build TRANSLATION_VARS macro for C which builds intermediate values.
 
@@ -810,10 +813,10 @@ def _build_translation_vars(table_id, vars):
         #define TRANSLATION_VARS(_v) \
          const double _temporary_Re = cbrt(_v.volume/_v.eccentricity/M_4PI_3)
     """
-    if vars:
+    if variables:
         # Leave comma off def since last def doesn't have comma
         defs = ["const double %s = %s" % (name, eq)
-                for name, eq in vars]
+                for name, eq in variables]
         return ("#define TRANSLATION_VARS(%s) \\\n  "%table_id
                 + "; \\\n  ".join(defs))
     else:
@@ -984,7 +987,7 @@ def make_source(model_info):
         raise ValueError("Unexpected function I%s for unoriented shape"%xy_mode)
     elif base_table.orientation_parameters and xy_mode not in ('qac', 'qabc'):
         if xy_mode == 'qxy':
-            logger.warn("oriented shapes should define Iqac or Iqabc")
+            logger.warning("oriented shapes should define Iqac or Iqabc")
         else:
             raise ValueError("Expected Iqac or Iqabc for oriented shape")
 
@@ -1098,7 +1101,7 @@ def make_source(model_info):
 
 
 def _kernels(kernel, call_iq, clear_iq, call_iqxy, clear_iqxy, name):
-    # type: ([str,str], str, str, str) -> List[str]
+    # type: (Dict[str, str], str, str, str, str, str) -> List[str]
     code = kernel[0]
     path = kernel[1].replace('\\', '/')
     iq = [
@@ -1137,13 +1140,13 @@ def _kernels(kernel, call_iq, clear_iq, call_iqxy, clear_iqxy, name):
 
 
 def load_kernel_module(model_name):
-    # type: (str) -> module
+    # type: (str) -> ModuleType
     """
     Return the kernel module named in *model_name*.
 
     If the name ends in *.py* then load it as a custom model using
-    :func:`sasmodels.custom.load_custom_kernel_module`, otherwise
-    load it from :mod:`sasmodels.models`.
+    :func:`.custom.__init__.load_custom_kernel_module`, otherwise
+    load it as a builtin from *sasmodels.models*.
     """
     if model_name.endswith('.py'):
         kernel_module = load_custom_kernel_module(model_name)
