@@ -70,6 +70,47 @@ typedef union {
 #if defined(MAGNETIC) && NUM_MAGNETIC > 0
 // ===== Helper functions for magnetism =====
 
+// vector algebra
+void SET_VEC(double *vector, double v0, double v1, double v2)
+{
+    vector[0] = v0;
+    vector[1] = v1;
+    vector[2] = v2;
+}
+
+void SCALE_VEC(double *vector, double a)
+{
+    vector[0] = a*vector[0];
+    vector[1] = a*vector[1];
+    vector[2] = a*vector[2];
+}
+
+void ADD_VEC(double *result_vec, double *vec1, double *vec2)
+{
+    result_vec[0] = vec1[0] + vec2[0];
+    result_vec[1] = vec1[1] + vec2[1];
+    result_vec[2] = vec1[2] + vec2[2];
+}
+
+static double SCALAR_VEC( double *vec1, double *vec2)
+{
+    return vec1[0] * vec2[0] + vec1[1] * vec2[1] + vec1[2] * vec2[2];
+}
+
+static double MAG_VEC( double *vec)
+{
+    return sqrt(SCALAR_VEC(vec,vec));
+}
+
+void ORTH_VEC(double *result_vec, double *vec1, double *vec2)
+{
+    double scale =  SCALAR_VEC(vec1,vec2) / SCALAR_VEC(vec2,vec2);
+    result_vec[0] = vec1[0] - scale * vec2[0];
+    result_vec[1] = vec1[1] - scale * vec2[1];
+    result_vec[2] = vec1[2] - scale * vec2[2];
+}
+
+
 // Return value restricted between low and high
 static double clip(double value, double low, double high)
 {
@@ -78,10 +119,11 @@ static double clip(double value, double low, double high)
 
 // Compute spin cross sections given in_spin and out_spin
 // To convert spin cross sections to sld b:
-//     uu * (sld - m_sigma_x);
-//     dd * (sld + m_sigma_x);
-//     ud * (m_sigma_y - 1j*m_sigma_z);
-//     du * (m_sigma_y + 1j*m_sigma_z);
+//     uu * (sld - m_perp_x);
+//     dd * (sld + m_perp_x);
+//     ud * (m_perp_y - 1j*m_perp_z);
+//     du * (m_perp_y + 1j*m_perp_z);
+//(x,y,z) is a local magnetic coordinate system. m_perp_x denotes the magnetic scattering vector along the polarisation and m_perp_y the component along the scattering vector, m_perpz is orthogonal to the others.
 // weights for spin crosssections: dd du real, ud real, uu, du imag, ud imag
 static void set_spin_weights(double in_spin, double out_spin, double weight[6])
 {
@@ -129,34 +171,59 @@ static void set_spin_weights(double in_spin, double out_spin, double weight[6])
 static double mag_sld(
   const unsigned int xs, // 0=dd, 1=du.real, 2=ud.real, 3=uu, 4=du.imag, 5=ud.imag
   const double qx, const double qy,
-  const double px, const double py,
+  const double cos_mspin, const double sin_mspin,
+  const double cos_mphi, const double sin_mphi,
   const double sld,
   const double mx, const double my, const double mz
 )
 {
+  double Mvector[3];
+  double Pvector[3]; 
+  double qvector[3];
+  double rhom[3]; 
+  double perpy[3];
+  double perpz[3];  
+  double Mperp[3];
+
+  const double qsq = sqrt(qx*qx + qy*qy); 
+  SET_VEC(qvector, qx / qsq, qy / qsq, 0);
+  SET_VEC(Mvector, mx, my, mz);
+  //const double px = sin_mspin * cos_mphi;
+  //const double py = sin_mspin * sin_mphi;
+  //const double pz = cos_mspin;
+  SET_VEC(Pvector, sin_mspin * cos_mphi, sin_mspin * sin_mphi, cos_mspin);
+  //two unit vectors spanning up the plane perpendicular to polarisation for SF scattering
+  SET_VEC(perpy, -sin_mphi, cos_mphi, 0);
+  SET_VEC(perpz, -cos_mspin * cos_mphi, -cos_mspin * sin_mphi, sin_mspin);
+  ORTH_VEC(Mperp, Mvector, qvector);
+
+
+
   if (xs < 4) {
-    const double perp = qy*mx - qx*my;
+
     switch (xs) {
       default: // keep compiler happy; condition ensures xs in [0,1,2,3]
-      case 0: // dd => sld - D M_perpx
-          return sld - px*perp;
-      case 1: // du.real => -D M_perpy
-          return py*perp;
-      case 2: // ud.real => -D M_perpy
-          return py*perp;
-      case 3: // uu => sld + D M_perpx
-          return sld + px*perp;
+      case 0: // dd => sld - D Pvector \cdot Mperp
+          return sld - SCALAR_VEC(Pvector, Mperp);
+      case 1: // du.real          
+          return SCALAR_VEC(perpy, Mperp);
+      case 2: // ud.real   
+          return SCALAR_VEC(perpy, Mperp);
+      case 3: // uu => sld + D Pvector \cdot Mperp
+          return sld + SCALAR_VEC(Pvector, Mperp); 
     }
   } else {
     if (xs== 4) {
-      return -mz;  // du.imag => +D M_perpz
-    } else { // index == 5
-      return +mz;  // ud.imag => -D M_perpz
+      return - SCALAR_VEC(perpz, Mperp);  // du.imag => - i  nperp \cdot MperpP
+    } else { // index == 5 
+      return + SCALAR_VEC(perpz, Mperp);   // du.imag => + i nperp \cdot MperpP
     }
   }
 }
 
 
+
+
 #endif
 
 // ===== Helper functions for orientation and jitter =====
@@ -348,11 +415,14 @@ void KERNEL_NAME(
   //     up_frac_i = values[NUM_PARS+2];
   //     up_frac_f = values[NUM_PARS+3];
   //     up_angle = values[NUM_PARS+4];
+  //     up_phi = values[NUM_PARS+5];
   // TODO: could precompute more magnetism parameters before calling the kernel.
   double xs_weights[8];  // uu, ud real, du real, dd, ud imag, du imag, fill, fill
   double cos_mspin, sin_mspin;
+  double cos_mphi, sin_mphi;
   set_spin_weights(values[NUM_PARS+2], values[NUM_PARS+3], xs_weights);
-  SINCOS(-values[NUM_PARS+4]*M_PI_180, sin_mspin, cos_mspin);
+  SINCOS(values[NUM_PARS+4]*M_PI_180, sin_mspin, cos_mspin);
+  SINCOS(values[NUM_PARS+5]*M_PI_180, sin_mphi, cos_mphi);
 #endif // MAGNETIC
 
   // ** Fill in the initial results **
@@ -404,7 +474,6 @@ void KERNEL_NAME(
 /*
 Based on the level of the loop, uses C preprocessor magic to construct
 level-specific looping variables, including these from loop level 3:
-
   int n3 : length of loop for mesh level 3
   int i3 : current position in the loop for level 3, which is calculated
        from a combination of pd_start, pd_stride[3] and pd_length[3].
@@ -414,42 +483,33 @@ level-specific looping variables, including these from loop level 3:
   double weight3 : the product of weights from levels 3 and up, computed
        as weight5*weight4*w3[i3].  Note that we need an outermost
        value weight5 set to 1.0 for this to work properly.
-
 After expansion, the loop struction will look like the following:
-
   // --- PD_INIT(4) ---
   const int n4 = pd_length[4];
   const int p4 = pd_par[4];
   pglobal const double *v4 = pd_value + pd_offset[4];
   pglobal const double *w4 = pd_weight + pd_offset[4];
   int i4 = (pd_start/pd_stride[4])%n4;  // position in level 4 at pd_start
-
   // --- PD_INIT(3) ---
   const int n3 = pd_length[3];
   ...
   int i3 = (pd_start/pd_stride[3])%n3;  // position in level 3 at pd_start
-
   PD_INIT(2)
   PD_INIT(1)
   PD_INIT(0)
-
   // --- PD_OUTERMOST_WEIGHT(5) ---
   const double weight5 = 1.0;
-
   // --- PD_OPEN(4,5) ---
   while (i4 < n4) {
     parameter[p4] = v4[i4];  // set the value for pd parameter 4 at this mesh point
     const double weight4 = w4[i4] * weight5;
-
     // from PD_OPEN(3,4)
     while (i3 < n3) {
       parameter[p3] = v3[i3];  // set the value for pd parameter 3 at this mesh point
       const double weight3 = w3[i3] * weight4;
-
       PD_OPEN(3,2)
       PD_OPEN(2,1)
       PD_OPEN(0,1)
-
       // ... main loop body ...
       APPLY_PROJECTION    // convert jitter values to spherical coords
       BUILD_ROTATION      // construct the rotation matrix qxy => qabc
@@ -457,25 +517,20 @@ After expansion, the loop struction will look like the following:
           FETCH_Q         // set qx,qy from the q input vector
           APPLY_ROTATION  // convert qx,qy to qa,qb,qc
           CALL_KERNEL     // F2 = Iqxy(qa, qb, qc, p1, p2, ...)
-
       ++step;  // increment counter representing position in dispersity mesh
-
       PD_CLOSE(0)
       PD_CLOSE(1)
       PD_CLOSE(2)
-
       // --- PD_CLOSE(3) ---
       if (step >= pd_stop) break;
       ++i3;
     }
     i3 = 0; // reset loop counter for next round through the loop
-
     // --- PD_CLOSE(4) ---
     if (step >= pd_stop) break;
     ++i4;
   }
   i4 = 0; // reset loop counter even though no more rounds through the loop
-
 */
 
 
@@ -742,27 +797,28 @@ PD_OUTERMOST_WEIGHT(MAX_PD)
         #if defined(MAGNETIC) && NUM_MAGNETIC > 0
           // Compute the scattering from the magnetic cross sections.
           double F2 = 0.0;
-          const double qsq = qx*qx + qy*qy;
+          const double qsq = qx * qx + qy * qy;
           if (qsq > 1.e-16) {
-            // TODO: what is the magnetic scattering at q=0
-            const double px = (qy*cos_mspin + qx*sin_mspin)/qsq;
-            const double py = (qy*sin_mspin - qx*cos_mspin)/qsq;
+            // TODO: what is the magnetic scattering at q = 0
+            //const double px = sin_mspin * cos_mphi;
+            //const double py = sin_mspin * sin_mphi;
+            //const double pz = cos_mspin;
 
             // loop over uu, ud real, du real, dd, ud imag, du imag
-            for (unsigned int xs=0; xs<6; xs++) {
+            for (unsigned int xs = 0; xs < 6; xs++) {
               const double xs_weight = xs_weights[xs];
               if (xs_weight > 1.e-8) {
                 // Since the cross section weight is significant, set the slds
                 // to the effective slds for this cross section, call the
                 // kernel, and add according to weight.
-                for (int sk=0; sk<NUM_MAGNETIC; sk++) {
-                  const int32_t mag_index = NUM_PARS+5 + 3*sk;
+                for (int sk = 0; sk<NUM_MAGNETIC; sk++) {
+                  const int32_t mag_index = NUM_PARS + 6 + 3 * sk;
                   const int32_t sld_index = slds[sk];
                   const double mx = values[mag_index];
-                  const double my = values[mag_index+1];
-                  const double mz = values[mag_index+2];
+                  const double my = values[mag_index + 1];
+                  const double mz = values[mag_index + 2];
                   local_values.vector[sld_index] =
-                    mag_sld(xs, qx, qy, px, py, values[sld_index+2], mx, my, mz);
+                    mag_sld(xs, qx, qy, cos_mspin, sin_mspin, cos_mphi, sin_mphi, values[sld_index + 2], mx, my, mz);
 //if (q_index==0) printf("%d: (qx,qy)=(%g,%g) xs=%d sld%d=%g p=(%g,%g) m=(%g,%g,%g)\n",
 //  q_index, qx, qy, xs, sk, local_values.vector[sld_index], px, py, mx, my, mz);
                 }
